@@ -1,12 +1,12 @@
 import os
 from pathlib import Path
-from typing import List, Mapping, Optional, IO, Union, Dict
+from typing import List, Mapping, Optional, Union, Dict
 from collections import OrderedDict
 
 from jinja2 import Environment, PackageLoader, select_autoescape
 
 from vespa.json_serialization import ToJson, FromJson
-from vespa.ml import ModelConfig, BertModelConfig, TextTask
+from vespa.query import QueryModel
 
 
 class HNSW(ToJson, FromJson["HNSW"]):
@@ -78,9 +78,14 @@ class Field(ToJson, FromJson["Field"]):
         """
         Create a Vespa field.
 
-        Check the `Vespa documentation <https://docs.vespa.ai/documentation/reference/schema-reference.html#field>`__`
+        Check the `Vespa documentation <https://docs.vespa.ai/en/reference/schema-reference.html#field>`__
         for more detailed information about fields.
 
+        Once we have an :class:`ApplicationPackage` instance containing a :class:`Schema` and a :class:`Document`,
+        we usually want to add fields so that we can store our data in a structured manner.
+        We can accomplish that by creating :class:`Field` instances
+        and adding those to the :class:`ApplicationPackage` instance via :class:`Schema` and :class:`Document` methods.
+
         :param name: Field name.
         :param type: Field data type.
         :param indexing: Configures how to process data of a field during indexing.
@@ -243,7 +248,7 @@ class Document(ToJson, FromJson["Document"]):
         """
         Create a Vespa Document.
 
-        Check the `Vespa documentation <https://docs.vespa.ai/documentation/documents.html>`__`
+        Check the `Vespa documentation <https://docs.vespa.ai/en/documents.html>`__
         for more detailed information about documents.
 
         :param fields: A list of :class:`Field` to include in the document's schema.
@@ -314,7 +319,7 @@ class FieldSet(ToJson, FromJson["FieldSet"]):
         Create a Vespa field set.
 
         A fieldset groups fields together for searching. Check the
-        `Vespa documentation <https://docs.vespa.ai/documentation/reference/schema-reference.html#fieldset>`__`
+        `Vespa documentation <https://docs.vespa.ai/en/reference/schema-reference.html#fieldset>`__
         for more detailed information about field sets.
 
         :param name: Name of the fieldset
@@ -360,7 +365,7 @@ class Function(ToJson, FromJson["Function"]):
 
         Define a named function that can be referenced as a part of the ranking expression, or (if having no arguments)
         as a feature. Check the
-        `Vespa documentation <https://docs.vespa.ai/documentation/reference/schema-reference.html#function-rank>`__`
+        `Vespa documentation <https://docs.vespa.ai/en/reference/schema-reference.html#function-rank>`__`
         for more detailed information about rank functions.
 
         :param name: Name of the function.
@@ -434,11 +439,11 @@ class SecondPhaseRanking(ToJson, FromJson["SecondPhaseRanking"]):
         Create a Vespa second phase ranking configuration.
 
         This is the optional reranking performed on the best hits from the first phase. Check the
-        `Vespa documentation <https://docs.vespa.ai/documentation/reference/schema-reference.html#secondphase-rank>`__`
+        `Vespa documentation <https://docs.vespa.ai/en/reference/schema-reference.html#secondphase-rank>`__`
         for more detailed information about second phase ranking configuration.
 
         :param expression: Specify the ranking expression to be used for second phase of ranking. Check also the
-            `Vespa documentation <https://docs.vespa.ai/documentation/reference/ranking-expressions.html>`__`
+            `Vespa documentation <https://docs.vespa.ai/en/reference/ranking-expressions.html>`__`
             for ranking expression.
         :param rerank_count: Specifies the number of hits to be reranked in the second phase. Default value is 100.
 
@@ -491,23 +496,23 @@ class RankProfile(ToJson, FromJson["RankProfile"]):
 
         Rank profiles are used to specify an alternative ranking of the same data for different purposes, and to
         experiment with new rank settings. Check the
-        `Vespa documentation <https://docs.vespa.ai/documentation/reference/schema-reference.html#rank-profile>`__`
+        `Vespa documentation <https://docs.vespa.ai/en/reference/schema-reference.html#rank-profile>`__
         for more detailed information about rank profiles.
 
         :param name: Rank profile name.
         :param first_phase: The config specifying the first phase of ranking.
-            `More info <https://docs.vespa.ai/documentation/reference/schema-reference.html#firstphase-rank>`__`
+            `More info <https://docs.vespa.ai/en/reference/schema-reference.html#firstphase-rank>`__`
             about first phase ranking.
         :param inherits: The inherits attribute is optional. If defined, it contains the name of one other
             rank profile in the same schema. Values not defined in this rank profile will then be inherited.
         :param constants: Dict of constants available in ranking expressions, resolved and optimized at
             configuration time.
-            `More info <https://docs.vespa.ai/documentation/reference/schema-reference.html#constants>`__`
+            `More info <https://docs.vespa.ai/en/reference/schema-reference.html#constants>`__`
             about constants.
         :param functions: Optional list of :class:`Function` representing rank functions to be included in the rank
             profile.
         :param summary_features: List of rank features to be included with each hit.
-            `More info <https://docs.vespa.ai/documentation/reference/schema-reference.html#summary-features>`__`
+            `More info <https://docs.vespa.ai/en/reference/schema-reference.html#summary-features>`__`
             about summary features.
         :param second_phase: Optional config specifying the second phase of ranking.
             See :class:`SecondPhaseRanking`.
@@ -632,7 +637,7 @@ class OnnxModel(ToJson, FromJson["OnnxModel"]):
 
         Vespa has support for advanced ranking models through it’s tensor API. If you have your model in the ONNX
         format, Vespa can import the models and use them directly. Check the
-        `Vespa documentation <https://docs.vespa.ai/documentation/onnx.html>`__`
+        `Vespa documentation <https://docs.vespa.ai/en/onnx.html>`__`
         for more detailed information about field sets.
 
         :param model_name: Unique model name to use as id when referencing the model.
@@ -716,7 +721,7 @@ class Schema(ToJson, FromJson["Schema"]):
         """
         Create a Vespa Schema.
 
-        Check the `Vespa documentation <https://docs.vespa.ai/documentation/schemas.html>`__`
+        Check the `Vespa documentation <https://docs.vespa.ai/en/schemas.html>`__
         for more detailed information about schemas.
 
         :param name: Schema name.
@@ -960,9 +965,13 @@ class QueryProfileType(ToJson, FromJson["QueryProfileType"]):
         """
         Create a Vespa Query Profile Type.
 
-        Check the `Vespa documentation <https://docs.vespa.ai/documentation/query-profiles.html#query-profile-types>`__`
+        Check the `Vespa documentation <https://docs.vespa.ai/en/query-profiles.html#query-profile-types>`__
         for more detailed information about query profile types.
 
+        An :class:`ApplicationPackage` instance comes with a default :class:`QueryProfile` named `default`
+        that is associated with a :class:`QueryProfileType` named `root`,
+        meaning that you usually do not need to create those yourself, only add fields to them when required.
+
         :param fields: A list of :class:`QueryTypeField`.
 
         >>> QueryProfileType(
@@ -1068,9 +1077,14 @@ class QueryProfile(ToJson, FromJson["QueryProfile"]):
         """
         Create a Vespa Query Profile.
 
-        Check the `Vespa documentation <https://docs.vespa.ai/documentation/query-profiles.html>`__`
+        Check the `Vespa documentation <https://docs.vespa.ai/en/query-profiles.html>`__
         for more detailed information about query profiles.
 
+        A :class:`QueryProfile` is a named collection of query request parameters given in the configuration.
+        The query request can specify a query profile whose parameters will be used as parameters of that request.
+        The query profiles may optionally be type checked.
+        Type checking is turned on by referencing a :class:`QueryProfileType` from the query profile.
+
         :param fields: A list of :class:`QueryField`.
 
         >>> QueryProfile(fields=[QueryField(name="maxHits", value=1000)])
@@ -1113,6 +1127,36 @@ class QueryProfile(ToJson, FromJson["QueryProfile"]):
         )
 
 
+class ModelConfig(object):
+    def __init__(self, model_id) -> None:
+        self.model_id = model_id
+
+    def onnx_model(self):
+        raise NotImplementedError
+
+    def query_profile_type_fields(self):
+        raise NotImplementedError
+
+    def document_fields(self, document_field_indexing):
+        raise NotImplementedError
+
+    def rank_profile(self, include_model_summary_features, **kwargs):
+        raise NotImplementedError
+
+
+class Task(object):
+    def __init__(
+        self,
+        model_id: str,
+    ):
+        """
+        Base class for ML Tasks.
+
+        :param model_id: Id used to identify the model on Vespa applications.
+        """
+        self.model_id = model_id
+
+
 class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
     def __init__(
         self,
@@ -1123,14 +1167,18 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         stateless_model_evaluation: bool = False,
         create_schema_by_default: bool = True,
         create_query_profile_by_default: bool = True,
-        models: Optional[List[TextTask]] = None,
+        tasks: Optional[List[Task]] = None,
+        default_query_model: Optional[QueryModel] = None
     ) -> None:
         """
         Create a Vespa Application Package.
 
-        Check the `Vespa documentation <https://docs.vespa.ai/documentation/cloudconfig/application-packages.html>`__
+        Check the `Vespa documentation <https://docs.vespa.ai/en/cloudconfig/application-packages.html>`__
         for more detailed information about application packages.
 
+        An :class:`ApplicationPackage` instance comes with a default :class:`Schema`
+        that contains a default :class:`Document`
+
         :param name: Application name.
         :param schema: List of :class:`Schema`s of the application. If `None`, an empty :class:`Schema` with the same name of the
             application will be created by default.
@@ -1143,7 +1191,8 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
             is provided in the `schema` argument.
         :param create_query_profile_by_default: Include a default :class:`QueryProfile` and :class:`QueryProfileType`
             in case it is not explicitly defined by the user in the `query_profile` and `query_profile_type` parameters.
-        :param models: List of models to be used in sequence classification tasks.
+        :param tasks: List of tasks to be served.
+        :param default_query_model: Optional QueryModel to be used as default for the application.
 
         The easiest way to get started is to create a default application package:
 
@@ -1170,7 +1219,8 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         self.model_ids = []
         self.model_configs = {}
         self.stateless_model_evaluation = stateless_model_evaluation
-        self.models = {} if not models else {model.model_id: model for model in models}
+        self.models = {} if not tasks else {model.model_id: model for model in tasks}
+        self.default_query_model = default_query_model
 
     @property
     def schemas(self) -> List[Schema]:
@@ -1201,13 +1251,13 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         for schema in schemas:
             self._schema.update({schema.name: schema})
 
-    def get_model(self, model_name: str):
+    def get_model(self, model_id: str):
         try:
-            return self.models[model_name]
+            return self.models[model_id]
         except KeyError:
             raise ValueError(
                 "Model named {} not defined in the application package.".format(
-                    model_name
+                    model_id
                 )
             )
 
@@ -1216,6 +1266,7 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         model_config: ModelConfig,
         schema=None,
         include_model_summary_features=False,
+        document_field_indexing=None,
         **kwargs
     ) -> None:
         """
@@ -1226,6 +1277,8 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         :param include_model_summary_features: True to include model specific summary features, such as
             inputs and outputs that are useful for debugging. Default to False as this requires an extra model
             evaluation when fetching summary features.
+        :param document_field_indexing: List of indexing attributes for the document fields required by the ranking
+            model.
         :param kwargs: Further arguments to be passed to RankProfile.
         :return: None
         """
@@ -1238,153 +1291,28 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
             raise ValueError("model_id must be unique: {}".format(model_id))
         self.model_ids.append(model_id)
         self.model_configs[model_id] = model_config
-
-        if isinstance(model_config, BertModelConfig):
-            self._add_bert_rank_profile(
-                model_config=model_config,
-                include_model_summary_features=include_model_summary_features,
-                schema=schema,
-                **kwargs
-            )
-        else:
-            raise ValueError("Unknown model configuration type")
-
-    def _add_bert_rank_profile(
-        self,
-        model_config: BertModelConfig,
-        include_model_summary_features,
-        schema=None,
-        doc_token_ids_indexing=None,
-        **kwargs
-    ) -> None:
-
-        model_id = model_config.model_id
-
         #
-        # export model
+        # Export ONNX model
         #
-        model_file_path = model_id + ".onnx"
-        model_config.export_to_onnx(output_path=model_file_path)
-
-        self.get_schema(schema).add_model(
-            OnnxModel(
-                model_name=model_id,
-                model_file_path=model_file_path,
-                inputs={
-                    "input_ids": "input_ids",
-                    "token_type_ids": "token_type_ids",
-                    "attention_mask": "attention_mask",
-                },
-                outputs={"output_0": "logits"},
-            )
-        )
-
+        self.get_schema(schema).add_model(model_config.onnx_model())
         #
-        # Add query profile type for query token ids
+        # Add query profile type fields
         #
-        self.query_profile_type.add_fields(
-            QueryTypeField(
-                name="ranking.features.query({})".format(
-                    model_config.query_token_ids_name
-                ),
-                type="tensor<float>(d0[{}])".format(
-                    int(model_config.actual_query_input_size)
-                ),
-            )
-        )
-
+        self.query_profile_type.add_fields(*model_config.query_profile_type_fields())
         #
         # Add field for doc token ids
         #
-        if not doc_token_ids_indexing:
-            doc_token_ids_indexing = ["attribute", "summary"]
         self.get_schema(schema).add_fields(
-            Field(
-                name=model_config.doc_token_ids_name,
-                type="tensor<float>(d0[{}])".format(
-                    int(model_config.actual_doc_input_size)
-                ),
-                indexing=doc_token_ids_indexing,
-            ),
+            *model_config.document_fields(
+                document_field_indexing=document_field_indexing
+            )
         )
-
         #
         # Add rank profiles
         #
-        constants = {"TOKEN_NONE": 0, "TOKEN_CLS": 101, "TOKEN_SEP": 102}
-        if "contants" in kwargs:
-            constants.update(kwargs.pop("contants"))
-
-        functions = [
-            Function(
-                name="question_length",
-                expression="sum(map(query({}), f(a)(a > 0)))".format(
-                    model_config.query_token_ids_name
-                ),
-            ),
-            Function(
-                name="doc_length",
-                expression="sum(map(attribute({}), f(a)(a > 0)))".format(
-                    model_config.doc_token_ids_name
-                ),
-            ),
-            Function(
-                name="input_ids",
-                expression="tokenInputIds({}, query({}), attribute({}))".format(
-                    model_config.input_size,
-                    model_config.query_token_ids_name,
-                    model_config.doc_token_ids_name,
-                ),
-            ),
-            Function(
-                name="attention_mask",
-                expression="tokenAttentionMask({}, query({}), attribute({}))".format(
-                    model_config.input_size,
-                    model_config.query_token_ids_name,
-                    model_config.doc_token_ids_name,
-                ),
-            ),
-            Function(
-                name="token_type_ids",
-                expression="tokenTypeIds({}, query({}), attribute({}))".format(
-                    model_config.input_size,
-                    model_config.query_token_ids_name,
-                    model_config.doc_token_ids_name,
-                ),
-            ),
-            Function(
-                name="logit0",
-                expression="onnx(" + model_id + ").logits{d0:0,d1:0}",
-            ),
-            Function(
-                name="logit1",
-                expression="onnx(" + model_id + ").logits{d0:0,d1:1}",
-            ),
-        ]
-        if "functions" in kwargs:
-            functions.extend(kwargs.pop("functions"))
-
-        summary_features = []
-        if include_model_summary_features:
-            summary_features.extend(
-                [
-                    "logit0",
-                    "logit1",
-                    "input_ids",
-                    "attention_mask",
-                    "token_type_ids",
-                ]
-            )
-        if "summary_features" in kwargs:
-            summary_features.extend(kwargs.pop("summary_features"))
-
         self.get_schema(schema).add_rank_profile(
-            RankProfile(
-                name=model_id,
-                constants=constants,
-                functions=functions,
-                summary_features=summary_features,
-                **kwargs
+            model_config.rank_profile(
+                include_model_summary_features=include_model_summary_features, **kwargs
             )
         )
 
@@ -1503,13 +1431,15 @@ class ModelServer(ApplicationPackage):
     def __init__(
         self,
         name: str,
-        models: Optional[List[TextTask]] = None,
+        tasks: Optional[List[Task]] = None,
     ):
         """
         Create a Vespa stateless model evaluation server.
+
         A Vespa stateless model evaluation server is a simplified Vespa application without content clusters.
+
         :param name: Application name.
-        :param models: List of models to be used in sequence classification tasks.
+        :param tasks: List of tasks to be served.
         """
         super().__init__(
             name=name,
@@ -1519,7 +1449,7 @@ class ModelServer(ApplicationPackage):
             stateless_model_evaluation=True,
             create_schema_by_default=False,
             create_query_profile_by_default=False,
-            models=models,
+            tasks=tasks,
         )
 
     @staticmethod
