@@ -2,7 +2,12 @@ import os
 import asyncio
 import shutil
 import json
-
+import unittest
+from pandas import DataFrame
+from cryptography.hazmat.primitives import serialization
+from vespa.application import Vespa
+from vespa.query import QueryModel, OR
+from vespa.gallery import TextSearch
 from vespa.deployment import VespaCloud
 from vespa.test_integration_docker import (
     TestApplicationCommon,
@@ -13,6 +18,61 @@ from vespa.test_integration_docker import (
 )
 
 
+class TestVespaKeyAndCertificate(unittest.TestCase):
+    def setUp(self) -> None:
+        self.app_package = create_msmarco_application_package()
+        self.vespa_cloud = VespaCloud(
+            tenant="vespa-team",
+            application="pyvespa-integration",
+            key_content=os.getenv("VESPA_CLOUD_USER_KEY").replace(r"\n", "\n"),
+            application_package=self.app_package,
+        )
+        self.disk_folder = os.path.join(os.getenv("WORK_DIR"), "sample_application")
+        self.instance_name = "msmarco"
+        self.app = self.vespa_cloud.deploy(
+            instance=self.instance_name, disk_folder=self.disk_folder
+        )
+
+    def test_key_cert_arguments(self):
+        #
+        # Write key and cert to different files
+        #
+        with open(os.path.join(self.disk_folder, "key_file.txt"), "w+") as file:
+            file.write(
+                self.vespa_cloud.data_key.private_bytes(
+                    serialization.Encoding.PEM,
+                    serialization.PrivateFormat.TraditionalOpenSSL,
+                    serialization.NoEncryption(),
+                ).decode("UTF-8")
+            )
+        with open(os.path.join(self.disk_folder, "cert_file.txt"), "w+") as file:
+            file.write(
+                self.vespa_cloud.data_certificate.public_bytes(
+                    serialization.Encoding.PEM
+                ).decode("UTF-8")
+            )
+        self.app = Vespa(
+            url=self.app.url,
+            key=os.path.join(self.disk_folder, "key_file.txt"),
+            cert=os.path.join(self.disk_folder, "cert_file.txt"),
+            application_package=self.app.application_package,
+        )
+        self.assertEqual(200, self.app.get_application_status().status_code)
+        self.assertDictEqual(
+            {
+                "pathId": "/document/v1/msmarco/msmarco/docid/1",
+                "id": "id:msmarco:msmarco::1",
+            },
+            self.app.get_batch(batch=[{"id": 1}])[0].json,
+        )
+
+    def tearDown(self) -> None:
+        self.app.delete_all_docs(
+            content_cluster_name="msmarco_content", schema="msmarco"
+        )
+        shutil.rmtree(self.disk_folder, ignore_errors=True)
+
+
 class TestMsmarcoApplication(TestApplicationCommon):
     def setUp(self) -> None:
         self.app_package = create_msmarco_application_package()
@@ -47,7 +107,9 @@ class TestMsmarcoApplication(TestApplicationCommon):
         # The port should not be 4443, see https://jira.vzbuilders.com/browse/VESPA-21365
         self.get_model_endpoints_when_no_model_is_available(
             app=self.app,
-            expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(self.app_package.name, self.instance_name),
+            expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(
+                self.app_package.name, self.instance_name
+            ),
         )
 
     def test_prediction_when_model_not_defined(self):
@@ -93,6 +155,15 @@ class TestMsmarcoApplication(TestApplicationCommon):
             fields_to_update=self.fields_to_update,
         )
 
+    def test_batch_operations_default_mode_with_one_schema(self):
+        self.batch_operations_default_mode_with_one_schema(
+            app=self.app,
+            schema_name=self.app_package.name,
+            fields_to_send=self.fields_to_send,
+            expected_fields_from_get_operation=self.fields_to_send,
+            fields_to_update=self.fields_to_update,
+        )
+
     def tearDown(self) -> None:
         self.app.delete_all_docs(
             content_cluster_name="msmarco_content", schema="msmarco"
@@ -159,7 +230,9 @@ class TestCord19Application(TestApplicationCommon):
         # The port should not be 4443, see https://jira.vzbuilders.com/browse/VESPA-21365
         self.get_model_endpoints_when_no_model_is_available(
             app=self.app,
-            expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(self.app_package.name, self.instance_name),
+            expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(
+                self.app_package.name, self.instance_name
+            ),
         )
 
     def test_prediction_when_model_not_defined(self):
@@ -207,6 +280,15 @@ class TestCord19Application(TestApplicationCommon):
             fields_to_update=self.fields_to_update,
         )
 
+    def test_batch_operations_default_mode_with_one_schema(self):
+        self.batch_operations_default_mode_with_one_schema(
+            app=self.app,
+            schema_name=self.app_package.name,
+            fields_to_send=self.fields_to_send,
+            expected_fields_from_get_operation=self.expected_fields_from_get_operation,
+            fields_to_update=self.fields_to_update,
+        )
+
     def test_bert_model_input_and_output(self):
         self.bert_model_input_and_output(
             app=self.app,
@@ -272,7 +354,9 @@ class TestQaApplication(TestApplicationCommon):
         # The port should not be 4443, see https://jira.vzbuilders.com/browse/VESPA-21365
         self.get_model_endpoints_when_no_model_is_available(
             app=self.app,
-            expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(self.app_package.name, self.instance_name),
+            expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(
+                self.app_package.name, self.instance_name
+            ),
         )
 
     def test_prediction_when_model_not_defined(self):
@@ -335,6 +419,59 @@ class TestQaApplication(TestApplicationCommon):
         shutil.rmtree(self.disk_folder, ignore_errors=True)
 
 
+class TestGalleryTextSearch(unittest.TestCase):
+    def setUp(self) -> None:
+        #
+        # Create application
+        #
+        self.app_package = TextSearch(id_field="id", text_fields=["title", "body"])
+        #
+        # Deploy application
+        #
+        self.vespa_cloud = VespaCloud(
+            tenant="vespa-team",
+            application="pyvespa-integration",
+            key_content=os.getenv("VESPA_CLOUD_USER_KEY").replace(r"\n", "\n"),
+            application_package=self.app_package,
+        )
+        self.disk_folder = os.path.join(os.getenv("WORK_DIR"), "sample_application")
+        self.instance_name = "text-search"
+        self.app = self.vespa_cloud.deploy(
+            instance=self.instance_name, disk_folder=self.disk_folder
+        )
+        #
+        # Create a sample data frame
+        #
+        records = [
+            {
+                "id": idx,
+                "title": "This doc is about {}".format(x),
+                "body": "There is so much to learn about {}".format(x),
+            }
+            for idx, x in enumerate(
+                ["finance", "sports", "celebrity", "weather", "politics"]
+            )
+        ]
+        df = DataFrame.from_records(records)
+        #
+        # Feed application
+        #
+        self.app.feed_df(df)
+
+    def test_query(self):
+        result = self.app.query(
+            query="what is finance?", query_model=QueryModel(match_phase=OR())
+        )
+        for hit in result.hits:
+            self.assertIn("fields", hit)
+
+    def tearDown(self) -> None:
+        self.app.delete_all_docs(
+            content_cluster_name="text_search_content", schema="text_search"
+        )
+        shutil.rmtree(self.disk_folder, ignore_errors=True)
+
+
 class TestSequenceClassification(TestApplicationCommon):
     def setUp(self) -> None:
         self.app_package = create_sequence_classification_task()
